function [x, skip] = ipy_read_eventlist(events_txt)

% [X,SKIP] = IPY_READ_EVENTLIST(EVENTS_TXT)
%
% Read a "raw" IPY event list file ("events.txt") and perform additional
% event selection by dropping dupplicate events, by dropping events weaker than 
% min ratio  (= 10), and dropping events where we do not have at least four 
% __consequtive__ valid data points in the 0.24 sec resolution time series
% (this is done by requiring event duration to be at least 0.5 sec). Also,
% correct RCS and effective diameter for the incorrect antenna gains that 
% are used in the analysis. (The analysis assumes that both TX and RX have
% been via the ESR 42m antenna, while actually the RX has been with the
% 32m antenna.) 
%
% The returned structure X contains the following fields (each is a vector)
%
%  DN       Matlab date number of the event
%  TX       Transmission power MW
%  AZ       Azimuth degr, N=0, E = 90
%  EL       Elevation degr
%  RT       Max Ratio ( = estimate of sqrt(ENR)) 
%  RG       Range km
%  RR       Range rate (km/s), computed from r = r(t)
%  VD       Doppler velocity (km/s), positive away from the radar
%  AD       Radial acceleration from vD = vD(t), km/s^2.
%  DI       Effective diameter cm, computed from RCS using simplified model
%  CS       RCS cm^2
%  TS       TX sample power / Noise power, dB, arbitrary reference point.
%  EN       Event number in the raw eventlist file
%  ED       Event duration, seconds
%  TP       TX sample power dB, arbitrary reference point
%  MT       Flag, set if multitarget suspected
%
% The vector SKIP is a list of events numbers of the raw events that 
% are filtered out here.
%
% Example
%
%  >> [x,skip] = ipy_read_eventlist('~/desktop/ipy_good_days/070821/events.txt');
%  >> length(skip)
%  ans =
%     289
%  >> length(x.dn)
%  ans =
%     944
%
% 4-Sep-2007 Jussi Markkanen.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    repair_rcs = 1;
    min_rt = 10.0; 
    min_ed = 0.5;

    if ~exist (events_txt,'file')
        error ([events_txt ' not found'])
    end

    [x, msg,ndup] = sd_read_eventlist (events_txt);
    if msg 
        error(msg) 
    end
    n_raw = length(x.dn) + ndup;

    if repair_rcs    
        x.cs = x.cs * 1.9055;
        x.di = sd_sigma2diam(x.cs*1E-4,0.6)*1E2;
    end

    good = find ( x.rt >= min_rt & x.ed > min_ed);
    x = sd_select_events (x,good);

    nskipped = n_raw - length(x.dn);

    if nargout > 1
        all_events = 1:n_raw;
        good_events = x.en(:);
        skip = setdiff(all_events,good_events);
    end
