function [X,msg,ndup] = sd_read_eventlist (file)

% X = SD_READ_EVENTLIST (FILE)
%
% Read the space debris analysis result file such as 
% events.txt to a structure.
%
% See also sd_save_an_result.m.


% VERSIONS BEFORE 1.0:
% ===================
%                           7     8    9      10   11        12      13      14        15       16      17      18       
%         TM                PW    AZ   EL     RT   RG        RR      VD      AD        DI       CS      TS      EN
%2005 9  6  19 36 45.090  1.30 180.0 90.0  257.5   695.646   0.093   0.014   70.89     7.27     41.523  16.45    1  % U20050906_193640_863
%-----------------------------------------------------------------------------------------------------------------
%
% VERSION 1.0:
% ===========
%                           7     8     9      10    11        12      13      14        15     16      17         18  19        20
%         TM                TX    AZ    EL     RT    RG        RR      VD      AD        DI     CS      TS         EN  ED        TP
%2005 11 10  00 02 11.611   0.80  182.1 81.6   60.1  1013.655  0.661   0.657   44.37     6.86   0.743   32.43      1   14.005    74.81   % U20051110_000159_405
%------------------------------------------------------------------------------------------------------------------------------------
%
%
% VERSION 1.1 (accel AD now in km/s):
% ===========
%                           7    8     9    10    11       12    13    14    15    16    17     18  19     20
%        TM                 TX   AZ    EL   RT    RG       RR    VD    AD    DI    CS    TS     EN  ED     TP
% 2005 11 18  00 27 14.228  1.80 184.9 77.4 258.4 1534.718 0.830 0.856 0.026 32.82 2.927 18.79  5   6.8850 67.49   % U20051118_002711_188
%-------------------------------------------------------------------------------------------------------------------------------------
%

% VERSION 1.2 (RCS now in log10 (RCS / 0.1 mm^2)
% ===========
%                             7     8     9     10        11      12      13      14       15      16      17     18        19       20
%           TM                TX    AZ   EL     RT        RG      RR      VD      AD       DI      CS      TS     EN        ED       TP
%   -------------------------------------------------------------------------------------------------------------------------------------
%
% 2006  3 17  13 41 06.563  0.95 182.1 81.6   34.0   739.464   0.693   0.667   0.080     5.66   0.244   36.57      1    7.5000    82.45   % U20060317_134102_809
%
% VERSION 1.3 (MT -- multi-target -- flag added).
%============
%                             7     8     9     10        11      12      13      14       15      16      17     18        19       20   21
%  -----------------------------------------------------------------------------------------------------------------------------------------
%           TM                TX    AZ   EL     RT        RG      RR      VD      AD       DI      CS      TS     EN        ED       TP   MT
%  -----------------------------------------------------------------------------------------------------------------------------------------
% 2007  3 17  00 01 37.542  0.85 182.1 81.6  139.7   942.747  -0.570  -0.631   0.047     8.16   4.195   28.72      1    4.3200    72.63    1  % U20070317_000130_341
%
% 21-Dec-2003 Jm
% 21-Dec-2004 Jm FILE can be cell array of names, in that case, X
%                is the merged list.
% 29-Nov-2005 Jm Support for event duration added. 
% 31-Jan-2006 Jm This header updated.
% 19-Mar-2006 Jm Support for eventlist version 1.2 added.
% 13-Apr-2006 Jm Bug fix. Acceleration conversion from km/s^2 to m/s^2 
%                was done in a wrong place, and "accumulated" when
%                there was more than one FILE.
% 13-Apr-2007 Jm Support for version 1.3. 
%  5-Jul-2007 Jm Added removal of duplicates, based on common range and time.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if nargin ~= 1
        error ('Wrong number of input arguments -- 1 expected')
    end
    
    msg = '';
    X = [];
    
    DN = 1:6;
    TX = 7;
    AZ = 8;
    EL = 9;
    RT = 10;
    RG = 11;
    RR = 12;
    VD = 13;
    AD = 14;     % acceleration from doppler velocity, m/s or km/s
    DI = 15;     % diam (cm)
    CS = 16;     % RCS   (returned in cm^2, even though in file it may be in log scale)
    TS = 17;     % tx snr   (returned in dB)
    EN = 18;     % event number
    
    ED = 19;     % event duration (returned in sec)     % from version 0.1 -->
    TP = 20;     % transmission sample power  (dB)      % from version 1.0 -->
    MT = 21;     % multi-target suspected               % from version 1.3 -->
    
    if ischar (file)
        FILE = cellstr(file);
    else
        FILE = file;
    end    
    
    for n = length(FILE)
        file = FILE{n};
        if ~exist (file,'file')
            msg = [file ' not found'];
            return
        end    
    end
    
    X.dn = []; 
    X.tx = [];
    X.az = [];
    X.el = [];
    X.rg = [];
    X.rr = [];
    X.vd = [];
    X.ad = [];
    X.di = [];
    X.cs = [];
    X.rt = [];
    X.ts = [];
    X.en = [];
    X.ed = [];
    X.tp = [];
    X.mt = [];
    
    for n = 1:length(FILE)
    
        file = FILE{n};
        x = load (file);
        [Nevents,Ntypes] = size(x); 
        if Ntypes <= 18
            VS = '0.0';     % last field is EN
        elseif Ntypes <= 19 
            VS = '0.1';     % last field is ED
        else
            VS = get_version (file);    % all fields up to TX; VS encoded into file 
        end
        vs = str2num(VS);
    
        K = isnan(x(:,VD));
        x(K,:) = [];
    
        dn = datenum ( x(:,1), x(:,2), x(:,3), x(:,4), x(:,5), x(:,6) );    

        [x,dn,ndup] = remove_duplicates (x,dn,RG);

        %fprintf('ndup = %d/%d\n',ndup,Nevents);     
     
        X.dn = [X.dn;dn]; 
        X.tx = [X.tx; x(:,TX)];
        X.az = [X.az; x(:,AZ)];
        X.el = [X.el; x(:,EL)];
        X.rg = [X.rg; x(:,RG)];
        X.rr = [X.rr; x(:,RR)];
        X.vd = [X.vd; x(:,VD)];
        X.di = [X.di; x(:,DI)];
    
        if vs < 1
            X.cs = [X.cs; x(:,CS) ];
        elseif vs >= 1.2
            % convert from log10 0.1 mm^2 to cm^2, linear scale
            X.cs = [X.cs; 1E-3*(10 .^ x(:,CS))];
        else
            % convert from log10 cm^2 to linear scale
            X.cs = [X.cs; 10 .^ x(:,CS)];
        end
    
        if  vs < 1
            % convert from linear to dB
            X.ts = [ X.ts; 10 * log10(x(:,TS))];
        else
            X.ts = [X.ts; x(:,TS)];
        end
    
        if vs > 1
            % Convert acc to m/s^2
            X.ad = [X.ad; x(:,AD)*1000];
        else
            X.ad = [X.ad; x(:,AD)];
        end
    
    %    if max(X.ad) > 10000
    %        keyboard
    %    end
    
        X.en = [X.en; x(:,EN)];
        X.rt = [X.rt; x(:,RT)];
        if vs > 0
            X.ed = [X.ed; x(:,ED)];
        end
        if vs >= 1
            X.tp = [X.tp; x(:,TP)];
        end
        if vs >= 1.3
            X.mt = [X.mt; x(:,MT)];
        end
    end
end


function VS = get_version (file)
%------------------------------------
%
%------------------------------------
    fid = fopen (file,'r');
    if fid < 2
        error (['could not open ' file]);
    end
    
    tline = strtrim(fgetl(fid));
    if strcmp (tline(1:10),'%! VERSION')
        VS = strtrim (tline(11:end));
    else
        VS = '0.5';
    end
    
    fclose(fid);
end


function [x,dn, nbad] = remove_duplicates (x,dn,RG)
%--------------------------------------------
%
%--------------------------------------------
    depth = 5;
    rtol = 1;
    second = 1/(24*3600);
    dntol = 1*second;

    [nrows,ncolumns] = size(x);
    if nrows <= depth, return, end

    badlist = zeros(1,ceil(nrows/10));
    nbad = 0;

    n1 = 1;
    n2 = depth-1;
    for n = depth:nrows
        K = find(  abs(dn(n) - dn(n1:n2)) < dntol ...
                 & abs(x(n,RG) - x(n1:n2,RG)) < rtol );
        if ~isempty(K)

            nbad = nbad+1;
            badlist(nbad) = n;        
        end
        n1 = n1+1;        
        n2 = n2+1;
    end
    x(badlist(1:nbad),:) = [];
    dn(badlist(1:nbad)) = [];
end

